package fi.jyu.mit.fxgui;


import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.ObservableList;
import javafx.scene.control.ComboBox;

/**
 * A custom ComboBox control that supports easy addition of items in Scene
 * Builder.
 * 
 * @author vesal modified by Tero Paavolainen
 * @version 13.1.2017
 * @param <T> Mitä luokkaa tallennetaan
 *
 */
public class ComboBoxChooser<T> extends ComboBox<StringAndObject<T>> implements Chooser<T> {
	private StringProperty rivit = new SimpleStringProperty();
	/**
	 * Is the first to be selected on each run
	 */
	protected BooleanProperty selectFirst = new SimpleBooleanProperty(true); 
	
	/**
	 * Initializes the control.
	 */
	public ComboBoxChooser() {
		super();
		if(selectFirst.get())
		   this.getSelectionModel().selectFirst();
	}

	
    private void setRows(String[] strings) {
        ObservableList<StringAndObject<T>> objects = getItems();
        objects.clear();
        for (String string : strings) {
            add(string, null);
        }
        if(selectFirst.get())
            this.getSelectionModel().selectFirst();
    }
    
    
	/**
	 * Sets the rows.
	 * @param jono A multiline string, each line representing a single row.
	 */
	@Override
    public void setRivit(String jono) {
		this.rivit.set(jono);
		String[] strings = jono.split("\n"); // erotinMerkki.get());
		setRows(strings);
	}

	
    @Override
    public void setRivit(String[] rivit) {
        StringBuilder sb = new StringBuilder();
        String sep = "";
        for (String s: rivit) { sb.append(sep).append(s); sep = "\n"; }
        this.rivit.set(sb.toString());
        setRows(rivit);
    }


	/**
	 * @return The contents of the Chooser as a multiline string.
	 */
	@Override
    public String getRivit() {
		return rivit.get();
	}

	
	/**
	 * @return The rows of the Chooser as a StringProperty.
	 */
	public StringProperty getRivitProperty() {
		return rivit;
	}
	
	
	/**
	 * Adds the given object with given name
	 * @param name objektin kohdalla näkyvä teksti
	 * @param object tallennettava olio
	 */
	@Override
    public void add(String name, T object){
	    String text = name;
        if (text != null)
            text = Functions.removeMnemonic(text);
		getItems().add(new StringAndObject<T>(text, object));
	}
	
	
	/**
	 * Addes the given object to the list
	 * @param object tallennettava olio
	 */
	@Override
    public void add(T object){
		add(null, object);
	}
	
	
	/**
	 * Adds example text to the list
	 * @param text teksti joka näytetään
	 */
	@Override
    public void addExample(String text){
		add(text, null);
	}
	
	
	/**
	 * Clears the ObservableList that holds the objects
	 */
	@Override
    public void clear(){
		getItems().clear();
	}


	@Override
	public T getSelectedObject() {
        StringAndObject<T> item = getSelectionModel().getSelectedItem();
        if ( item == null ) return null;
	    return item.getObject();
	}


	@Override
	public String getSelectedText() {
		StringAndObject<T> item = getSelectionModel().getSelectedItem();
	    if ( item == null ) return null;
		return item.getName();
	}
	
	@Override
    public int getSelectedIndex() {
        return getSelectionModel().getSelectedIndex();
	}
	
	
    @Override
    public int setSelectedIndex(int index) {
        int oldIndex = getSelectedIndex();
        getSelectionModel().select(index);
        return oldIndex;
    }
    
    /**
     * If index already selected and selected = false. Nothing will be selected.
     * Otherwise wont select anything with selected = false;
     */
    @Override
    public int setSelectedIndex(int index, boolean selected){
        if (selected){
           return setSelectedIndex(index); 
        }
        int old  = getSelectedIndex();
        if(index == old){
            setSelectedIndex(-1);
            return old;
        }
        return old;
    }
    
	
	/**
	 * Returns the value of property getSelectFirst
	 * @return if to be selected
	 */
	public boolean getSelectFirst(){
	    return selectFirst.get();
	}
	
	
	/**
	 * Sets the value of setFirst
	 * @param value new value
	 */
	public void setSelectFirst(boolean value){
	    selectFirst.set(value);
	}


    @Override
    public void addSelectionListener(Consumer<T> event) {
        this.getSelectionModel().selectedItemProperty().addListener(new ChangeListener<StringAndObject<T>>() {

            @Override
            public void changed(
                    ObservableValue<? extends StringAndObject<T>> ov,
                    StringAndObject<T> old, StringAndObject<T> newer) {
                if ( newer == null)
                    event.accept(null);
                else
                    event.accept(newer.getObject());
                
            }
      }    );
        
    }

    
    @Override 
    public List<T> getObjects() {
        List<T> result = new ArrayList<>();
        for (StringAndObject<T> item: getItems()) result.add(item.getObject());
        return result;
    }

}
